@@ -1,6 +1,6 @@
 // maingui.cpp
 
-// Copyright (C) 2012-2014 by Werner Lemberg.
+// Copyright (C) 2012-2015 by Werner Lemberg.
 //
 // This file is part of the ttfautohint library, and may only be used,
 // modified, and distributed under the terms given in `COPYING'.  By
@@ -42,25 +42,39 @@
 #define Tr(text) QCoreApplication::translate(TRDOMAIN, text)
 
 
-// the available script tags and its descriptions are directly extracted
-// from `ttfautohint-scripts.h'
-typedef struct Script_Names_
+typedef struct Tag_Names_
 {
   const char* tag;
   const char* description;
-} Script_Names;
+} Tag_Names;
 
+
+// the available script tags and its descriptions are directly extracted
+// from `ttfautohint-scripts.h'
 #undef SCRIPT
 #define SCRIPT(s, S, d, h, sc1, sc2, sc3) \
           {#s, d},
 
-const Script_Names script_names[] =
+const Tag_Names script_names[] =
 {
 #include <ttfautohint-scripts.h>
   {NULL, NULL}
 };
 
 
+// the available feature tags and its descriptions are directly extracted
+// from `ttfautohint-coverages.h'
+#undef COVERAGE
+#define COVERAGE(n, N, d, t, t1, t2, t3, t4) \
+          {#t, d},
+
+const Tag_Names feature_names[] =
+{
+#include <ttfautohint-coverages.h>
+  {NULL, NULL}
+};
+
+
 // used hotkeys:
 //   a: Add ttf&autohint Info
 //   b: Add TTFA info ta&ble
@@ -77,7 +91,7 @@ const Script_Names script_names[] =
 //   m: Hint Co&mposites
 //   n: Hint Set Range Mi&nimum
 //   o: &Output File
-//   p: Windows Comp&patibility
+//   p: Windows Com&patibility
 //   q: --
 //   r: &Run
 //   s: Fallback &Script
@@ -104,8 +118,10 @@ Main_GUI::Main_GUI(bool horizontal_layout,
                    bool adjust,
                    bool composites,
                    bool no,
+                   bool detailed,
                    const char* dflt,
                    const char* fallback,
+                   const char* suffix,
                    bool symb,
                    bool dh,
                    bool TTFA)
@@ -123,6 +139,8 @@ Main_GUI::Main_GUI(bool horizontal_layout,
   adjust_subglyphs(adjust),
   hint_composites(composites),
   no_info(no),
+  detailed_info(detailed),
+  family_suffix(suffix),
   symbol(symb),
   dehint(dh),
   TTFA_info(TTFA)
@@ -207,7 +225,7 @@ Main_GUI::about()
   QMessageBox::about(this,
                      tr("About TTFautohint"),
                      tr("<p>This is <b>TTFautohint</b> version %1<br>"
-                        " Copyright %2 2011-2014<br>"
+                        " Copyright %2 2011-2015<br>"
                         " by Werner Lemberg <tt>&lt;wl@gnu.org&gt;</tt></p>"
                         ""
                         "<p><b>TTFautohint</b> adds new auto-generated hints"
@@ -232,6 +250,7 @@ Main_GUI::browse_input()
                    tr("Open Input File"),
                    QDir::homePath(),
                    "");
+
   if (!file.isEmpty())
     input_line->setText(QDir::toNativeSeparators(file));
 }
@@ -586,6 +605,23 @@ Main_GUI::check_number_set()
 }
 
 
+void
+Main_GUI::check_family_suffix()
+{
+  QString text = family_suffix_line->text();
+  const char* s = qPrintable(text);
+
+  if (const char* pos = ::check_family_suffix(s))
+  {
+    statusBar()->setStyleSheet("color: red;");
+    statusBar()->showMessage(
+      tr("invalid character (use printable ASCII except %()/<>[]{})"));
+    family_suffix_line->setFocus(Qt::OtherFocusReason);
+    family_suffix_line->setCursorPosition(pos - s);
+  }
+}
+
+
 void
 Main_GUI::clear_status_bar()
 {
@@ -712,6 +748,8 @@ Main_GUI::open_files(const QString& input_name,
       return 0;
     }
   }
+  else
+    *control = NULL;
 
   return 1;
 }
@@ -758,7 +796,7 @@ struct GUI_Progress_Data
 #undef TRDOMAIN
 #define TRDOMAIN "GuiProgress"
 
-int
+static int
 gui_progress(long curr_idx,
              long num_glyphs,
              long curr_sfnt,
@@ -823,7 +861,7 @@ struct GUI_Error_Data
 #undef TRDOMAIN
 #define TRDOMAIN "GuiError"
 
-void
+static void
 gui_error(TA_Error error,
           const char* error_string,
           unsigned int errlinenum,
@@ -994,6 +1032,7 @@ again:
   dialog.setWindowModality(Qt::WindowModal);
 
   TA_Info_Func info_func = info;
+  TA_Info_Post_Func info_post_func = info_post;
   GUI_Progress_Data gui_progress_data = {-1, true, &dialog};
   GUI_Error_Data gui_error_data = {this, locale, output_name, control_name,
                                    &ignore_restrictions, false};
@@ -1003,10 +1042,10 @@ again:
 
   Info_Data info_data;
 
-  info_data.data = NULL; // must be deallocated after use
-  info_data.data_wide = NULL; // must be deallocated after use
-  info_data.data_len = 0;
-  info_data.data_wide_len = 0;
+  info_data.info_string = NULL; // must be deallocated after use
+  info_data.info_string_wide = NULL; // must be deallocated after use
+  info_data.info_string_len = 0;
+  info_data.info_string_wide_len = 0;
 
   info_data.control_name = qPrintable(fileinfo_control_file.fileName());
 
@@ -1025,6 +1064,11 @@ again:
                                 : increase_box->value();
   info_data.x_height_snapping_exceptions_string =
     qPrintable(x_height_snapping_exceptions_string);
+
+  info_data.family_suffix =
+    family_suffix_line->text().toLocal8Bit().constData();
+  info_data.family_data_head = NULL;
+
   info_data.fallback_stem_width = default_stem_width_box->isChecked()
                                   ? 0
                                   : stem_width_box->value();
@@ -1033,6 +1077,8 @@ again:
   info_data.adjust_subglyphs = adjust_box->isChecked();
   info_data.hint_composites = hint_box->isChecked();
   info_data.symbol = symbol_box->isChecked();
+  info_data.no_info = info_box->currentIndex() == 0;
+  info_data.detailed_info = info_box->currentIndex() == 2;
   info_data.dehint = dehint_box->isChecked();
   info_data.TTFA_info = TTFA_box->isChecked();
 
@@ -1043,7 +1089,7 @@ again:
           script_names[fallback_box->currentIndex()].tag,
           sizeof (info_data.fallback_script));
 
-  if (info_box->isChecked())
+  if (info_box->currentIndex())
   {
     int ret = build_version_string(&info_data);
     if (ret == 1)
@@ -1063,8 +1109,9 @@ again:
         QMessageBox::Ok,
         QMessageBox::Ok);
   }
-  else
-    info_func = NULL;
+
+  if (!*info_data.family_suffix)
+    info_post_func = NULL;
 
   if (info_data.symbol
       && info_data.fallback_stem_width
@@ -1091,7 +1138,7 @@ again:
                  "dw-cleartype-strong-stem-width,"
                  "progress-callback, progress-callback-data,"
                  "error-callback, error-callback-data,"
-                 "info-callback, info-callback-data,"
+                 "info-callback, info-post-callback, info-callback-data,"
                  "ignore-restrictions,"
                  "windows-compatibility,"
                  "adjust-subglyphs,"
@@ -1108,7 +1155,7 @@ again:
                  info_data.dw_cleartype_strong_stem_width,
                  gui_progress, &gui_progress_data,
                  gui_error, &gui_error_data,
-                 info_func, &info_data,
+                 info_func, info_post_func, &info_data,
                  ignore_restrictions,
                  info_data.windows_compatibility,
                  info_data.adjust_subglyphs,
@@ -1118,10 +1165,10 @@ again:
                  info_data.default_script, info_data.fallback_script,
                  info_data.symbol, info_data.dehint, info_data.TTFA_info);
 
-  if (info_box->isChecked())
+  if (info_box->currentIndex())
   {
-    free(info_data.data);
-    free(info_data.data_wide);
+    free(info_data.info_string);
+    free(info_data.info_string_wide);
   }
 
   fclose(input);
@@ -1189,52 +1236,128 @@ Main_GUI::create_layout(bool horizontal_layout)
   control_line = new Drag_Drop_Line_Edit(DRAG_DROP_ANY);
   control_button = new QPushButton(tr("Browse..."));
   control_label->setBuddy(control_line);
-  control_label->setToolTip(
-    tr("<p>An optional control instructions file to tweak hinting."
+  QString tooltip_string =
+    tr("<p>An optional control instructions file to tweak hinting"
+       " and to override glyph assignments to styles."
        "  This text file contains entries"
-       " of one of the following syntax forms<br>"
+       " of one of the following syntax forms"
+       " (with brackets indicating optional elements).<br>"
        "&nbsp;<br>"
+
+       "&nbsp;&nbsp;[&nbsp;<i>subfont-idx</i>&nbsp;]"
+       "&nbsp;&nbsp;<i>script</i>"
+       "&nbsp;&nbsp;<i>feature</i>"
+       "&nbsp;&nbsp;<tt>@</tt>&nbsp;<i>glyph-ids</i><br>"
+
        "&nbsp;&nbsp;[&nbsp;<i>subfont-idx</i>&nbsp;]"
        "&nbsp;&nbsp;<i>glyph-id</i>"
-       "&nbsp;&nbsp;<tt>l</tt>|<tt>r</tt>&nbsp;<i>points</i>"
+       "&nbsp;&nbsp;<tt>left</tt>&nbsp;|&nbsp;<tt>right</tt>&nbsp;<i>points</i>"
        "&nbsp;&nbsp;[&nbsp;<tt>(</tt><i>left-offset</i><tt>,"
          "</tt><i>right-offset</i><tt>)</tt>&nbsp;]<br>"
+
        "&nbsp;&nbsp;[&nbsp;<i>subfont-idx</i>&nbsp;]"
        "&nbsp;&nbsp;<i>glyph-id</i>"
-       "&nbsp;&nbsp;<tt>n</tt>&nbsp;<i>points</i><br>"
+       "&nbsp;&nbsp;<tt>nodir</tt>&nbsp;<i>points</i><br>"
+
        "&nbsp;&nbsp;[&nbsp;<i>subfont-idx</i>&nbsp;]"
        "&nbsp;&nbsp;<i>glyph-id</i>"
-       "&nbsp;&nbsp;<tt>p</tt>&nbsp;<i>points</i>"
-       "&nbsp;&nbsp;[&nbsp;<tt>x</tt>&nbsp;<i>shift</i>&nbsp;]"
-       "&nbsp;&nbsp;[&nbsp;<tt>y</tt>&nbsp;<i>shift</i>&nbsp;]"
+       "&nbsp;&nbsp;<tt>touch</tt>&nbsp;|&nbsp;<tt>point</tt>&nbsp;<i>points</i>"
+       "&nbsp;&nbsp;[&nbsp;<tt>xshift</tt>&nbsp;<i>shift</i>&nbsp;]"
+       "&nbsp;&nbsp;[&nbsp;<tt>yshift</tt>&nbsp;<i>shift</i>&nbsp;]"
        "&nbsp;&nbsp;<tt>@</tt>&nbsp;<i>ppems</i><br>"
        "&nbsp;<br>"
+
        "<i>subfont-idx</i> gives the subfont index in a TTC,"
        " <i>glyph-id</i> is a glyph name or index.<br>"
-       "<tt>l</tt> (<tt>r</tt>) creates one-point segments with direction"
-       " left (right), possibly having a width (in font units)"
+       "&nbsp;<br>"
+
+       "<i>script</i> and <i>feature</i> are four-letter tags"
+       " that define a style the <i>glyph-ids</i> are assigned to."
+       "  <i>glyph-ids</i> is a comma-separated list of"
+       " <i>glyph-id</i> values and value ranges.<br>");
+
+  tooltip_string += tr("Possible values for <i>script</i> are ");
+  const Tag_Names* sn = script_names;
+  for(;;)
+  {
+    tooltip_string += QString::fromLocal8Bit("<tt>%1</tt> (%2)")
+                                             .arg(sn->tag)
+                                             .arg(sn->description);
+    sn++;
+    if (sn->tag)
+      tooltip_string += ", ";
+    else
+    {
+      tooltip_string += ".<br>";
+      break;
+    }
+  }
+
+  tooltip_string += tr("Possible values for <i>feature</i> are ");
+  const Tag_Names* fn = feature_names;
+  for(;;)
+  {
+    tooltip_string += QString::fromLocal8Bit("<tt>%1</tt> (%2)")
+                                             .arg(fn->tag)
+                                             .arg(fn->description);
+    fn++;
+    if (fn->tag)
+      tooltip_string += ", ";
+    else
+    {
+      tooltip_string += ".<br>";
+      break;
+    }
+  }
+
+  tooltip_string += "&nbsp;<br>";
+
+  tooltip_string +=
+    tr("<tt>left</tt> (<tt>right</tt>) creates one-point segments"
+       " with direction left (right), possibly having a width (in font units)"
        " given by <i>left-offset</i> and <i>right-offset</i>"
        " relative to the corresponding points.<br>"
-       "<tt>n</tt> removes points from horizontal segments,"
+       "&nbsp;<br>"
+
+       "<tt>nodir</tt> removes points from horizontal segments,"
        " making them <i>weak</i> points.<br>"
-       "<tt>p</tt> defines delta exceptions"
-       " (to be applied <i>after</i> the final"
-       " <tt>IP</tt> bytecode instructions;"
-       " this usually fails in ClearType mode).<br>"
+       "&nbsp;<br>"
+
+       "<tt>touch </tt> (<tt>point</tt>) defines delta exceptions"
+       " to be applied before (after) the final"
+       " <tt>IUP</tt> bytecode instructions."
+       "  <tt>touch</tt> also touches points, making them <i>strong</i>."
+       "  In ClearType mode, <tt>point</tt> and <tt>xshift</tt>"
+       " have no effect.<br>"
+       "&nbsp;<br>"
+
        "x and y <i>shift</i> values are in the range [-1;1],"
        " rounded to multiples of 1/8px.<br>"
+
        "<i>points</i> and <i>ppems</i> are ranges for point indices"
        " and ppem values as with x&nbsp;height snapping exceptions.<br>"
+
+       "Keywords <tt>left</tt>, <tt>right</tt>, <tt>nodir</tt>,"
+       " <tt>point</tt>, <tt>touch</tt>, <tt>xshift</tt>, and"
+       " <tt>yshift</tt> can be abbreviated as <tt>l</tt>, <tt>r</tt>,"
+       " <tt>n</tt>, <tt>p</tt>, <tt>t</tt>, <tt>x</tt>, and <tt>y</tt>,"
+       " respectively.<br>"
+
        "Control instruction entries are separated"
        " by character&nbsp;<tt>;</tt> or by a newline.<br>"
-       "  A trailing character&nbsp;<tt>\\</tt> continues the current line"
+
+       "A trailing character&nbsp;<tt>\\</tt> continues the current line"
        " on the next line.<br>"
+
        "<tt>#</tt> starts a line comment, which gets ignored."
        "  Empty lines are ignored, too.</p>"
-       ""
+
        "Examples:<br>"
-       "&nbsp;&nbsp;<tt>Q l 38 (-70,20)</tt><br>"
-       "&nbsp;&nbsp;<tt>Adieresis p 3-6 y 0.25 @ 13</tt>"));
+       "&nbsp;&nbsp;<tt>cyrl sups @ one.sups-nine.sups, zero.sups</tt><br>"
+       "&nbsp;&nbsp;<tt>Q left 38 (-70,20)</tt><br>"
+       "&nbsp;&nbsp;<tt>Adieresis touch 3-6 yshift 0.25 @ 13</tt>");
+
+  control_label->setToolTip(tooltip_string);
   control_line->setCompleter(completer);
 
   //
@@ -1341,7 +1464,7 @@ Main_GUI::create_layout(bool horizontal_layout)
   increase_box = new QSpinBox;
   increase_label->setBuddy(increase_box);
   increase_label->setToolTip(
-    tr("For PPEM values in the range 5&nbsp;&lt; PPEM &lt;&nbsp;<i>n</i>,"
+    tr("For PPEM values in the range 6&nbsp;&le; PPEM &le;&nbsp;<i>n</i>,"
        " where <i>n</i> is the value selected by this spin box,"
        " round up the font's x&nbsp;height much more often than normally.<br>"
        "Use this if holes in letters like <i>e</i> get filled,"
@@ -1374,6 +1497,20 @@ Main_GUI::create_layout(bool horizontal_layout)
        " (meaning PPEM &le; 20 or PPEM &ge; 40)<br>"
        "&nbsp;&nbsp;<tt>-</tt> (meaning all possible PPEM values)"));
 
+  //
+  // family suffix
+  //
+  family_suffix_label = new QLabel(tr("Family Suffix&:"));
+  family_suffix_line = new Tooltip_Line_Edit;
+  family_suffix_label->setBuddy(family_suffix_line);
+  family_suffix_label->setToolTip(
+    tr("A string that gets appended to the family name entries"
+       " (name IDs 1, 4, 6, 16, and&nbsp;21)"
+       " in the font's <i>name</i> table.<br>"
+       "Mainly for testing purposes, enabling the operating system"
+       " to display fonts simultaneously that are hinted"
+       " with different <b>TTFautohint</b> parameters."));
+
   //
   // fallback stem width
   //
@@ -1440,15 +1577,20 @@ Main_GUI::create_layout(bool horizontal_layout)
     tr("If set, remove all hints from the font.<br>"
        "For testing only."));
 
-  info_box = new QCheckBox(tr("Add ttf&autohint Info"), this);
-  info_box->setToolTip(
-    tr("If switched on, information about <b>TTFautohint</b>"
-       " and its calling parameters are added to the version string(s)"
+  info_label = new QLabel(tr("ttf&autohint Info:"));
+  info_box = new QComboBox;
+  info_label->setBuddy(info_box);
+  info_label->setToolTip(
+    tr("Specify which information about <b>TTFautohint</b>"
+       " and its calling parameters gets added to the version string(s)"
        " (name ID&nbsp;5) in the <i>name</i> table."));
+  info_box->insertItem(0, tr("None"));
+  info_box->insertItem(1, tr("Version"));
+  info_box->insertItem(2, tr("Version and Parameters"));
 
   TTFA_box = new QCheckBox(tr("Add TTFA Info Ta&ble"), this);
   TTFA_box->setToolTip(
-    tr("If switched on, an SFNT table called <tt>TTFA</tt>"
+    tr("If switched on, an SFNT table called <i>TTFA</i>"
        " gets added to the output font,"
        " holding a dump of all parameters."
        "  In particular, it lists all control instructions."));
@@ -1609,8 +1751,12 @@ Main_GUI::create_vertical_layout()
   gui_layout->addWidget(hint_box, row++, 1);
   gui_layout->addWidget(symbol_box, row++, 1);
   gui_layout->addWidget(dehint_box, row++, 1);
-  gui_layout->addWidget(info_box, row++, 1);
+
+  gui_layout->addWidget(info_label, row, 0, Qt::AlignRight);
+  gui_layout->addWidget(info_box, row++, 1, Qt::AlignLeft);
   gui_layout->addWidget(TTFA_box, row++, 1);
+  gui_layout->addWidget(family_suffix_label, row, 0, Qt::AlignRight);
+  gui_layout->addWidget(family_suffix_line, row++, 1, Qt::AlignLeft);
 
   gui_layout->setRowMinimumHeight(row, 20); // XXX urgh, pixels...
   gui_layout->setRowStretch(row++, 1);
@@ -1738,8 +1884,12 @@ Main_GUI::create_horizontal_layout()
   gui_layout->addWidget(hint_box, row++, 4);
   gui_layout->addWidget(symbol_box, row++, 4);
   gui_layout->addWidget(dehint_box, row++, 4);
-  gui_layout->addWidget(info_box, row++, 4);
+
+  gui_layout->addWidget(info_label, row, 3, Qt::AlignRight);
+  gui_layout->addWidget(info_box, row++, 4, Qt::AlignLeft);
   gui_layout->addWidget(TTFA_box, row++, 4);
+  gui_layout->addWidget(family_suffix_label, row, 3, Qt::AlignRight);
+  gui_layout->addWidget(family_suffix_line, row++, 4, Qt::AlignLeft);
 
   gui_layout->setRowMinimumHeight(row, 20); // XXX urgh, pixels...
   gui_layout->setRowStretch(row++, 1);
@@ -1807,6 +1957,11 @@ Main_GUI::create_connections()
   connect(snapping_line, SIGNAL(textEdited(QString)), this,
           SLOT(clear_status_bar()));
 
+  connect(family_suffix_line, SIGNAL(editingFinished()), this,
+          SLOT(check_family_suffix()));
+  connect(family_suffix_line, SIGNAL(textEdited(QString)), this,
+          SLOT(clear_status_bar()));
+
   connect(default_stem_width_box, SIGNAL(clicked()), this,
           SLOT(check_default_stem_width()));
 
@@ -1885,6 +2040,7 @@ Main_GUI::set_defaults()
   }
 
   snapping_line->setText(x_height_snapping_exceptions_string);
+  family_suffix_line->setText(family_suffix);
 
   if (fallback_stem_width)
     stem_width_box->setValue(fallback_stem_width);
@@ -1904,10 +2060,14 @@ Main_GUI::set_defaults()
     symbol_box->setChecked(true);
   if (dehint)
     dehint_box->setChecked(true);
-  if (!no_info)
-    info_box->setChecked(true);
+  if (no_info)
+    info_box->setCurrentIndex(0);
+  else if (detailed_info)
+    info_box->setCurrentIndex(2);
+  else
+    info_box->setCurrentIndex(1);
   if (TTFA_info)
-    info_box->setChecked(true);
+    TTFA_box->setChecked(true);
 
   if (gray_strong_stem_width)
     gray_box->setChecked(true);
