@@ -1,6 +1,6 @@
 // maingui.h
 
-// Copyright (C) 2012-2014 by Werner Lemberg.
+// Copyright (C) 2012-2015 by Werner Lemberg.
 //
 // This file is part of the ttfautohint library, and may only be used,
 // modified, and distributed under the terms given in `COPYING'.  By
@@ -50,7 +50,8 @@ public:
            bool, int,
            const char*, int,
            bool, bool, bool,
-           bool, bool, const char*, const char*,
+           bool, bool, bool,
+           const char*, const char*, const char*,
            bool, bool, bool);
   ~Main_GUI();
 
@@ -73,6 +74,7 @@ private slots:
   void absolute_output();
   void absolute_control();
   void check_number_set();
+  void check_family_suffix();
   void clear_status_bar();
   void check_watch();
   void watch_files();
@@ -95,8 +97,10 @@ private:
   int adjust_subglyphs;
   int hint_composites;
   int no_info;
+  int detailed_info;
   int default_script_idx;
   int fallback_script_idx;
+  QString family_suffix;
   int symbol;
   int dehint;
   int TTFA_info;
@@ -175,6 +179,8 @@ private:
 
   QLabel* snapping_label;
   Tooltip_Line_Edit* snapping_line;
+  QLabel* family_suffix_label;
+  Tooltip_Line_Edit* family_suffix_line;
 
   QLabel* stem_width_label;
   QString stem_width_label_text_with_key;
@@ -189,7 +195,8 @@ private:
   QCheckBox* hint_box;
   QCheckBox* symbol_box;
   QCheckBox* dehint_box;
-  QCheckBox* info_box;
+  QLabel* info_label;
+  QComboBox* info_box;
   QCheckBox* TTFA_box;
 
   QCheckBox* watch_box;
