@@ -13,9 +13,11 @@
  */
 
 #include "ta.h"
+#include "taharfbuzz.h"
 
 #include <locale.h>
 #include <limits.h>
+#include <stdio.h>
 #include <errno.h>
 #include <ctype.h>
 #include <math.h>
@@ -24,6 +26,7 @@
 #include "llrb.h" /* a red-black tree implementation */
 #include "tacontrol-bison.h"
 
+
 Control*
 TA_control_new(Control_Type type,
                long font_idx,
@@ -44,7 +47,7 @@ TA_control_new(Control_Type type,
 
   control->font_idx = font_idx;
   control->glyph_idx = glyph_idx;
-  control->points = number_set_reverse(point_set);
+  control->points = number_set_normalize(number_set_reverse(point_set));
 
   switch (control->type)
   {
@@ -60,17 +63,22 @@ TA_control_new(Control_Type type,
   case Control_Segment_Left:
   case Control_Segment_Right:
     /* offsets */
-    control->x_shift = x_shift;
-    control->y_shift = y_shift;
+    control->x_shift = (int)x_shift;
+    control->y_shift = (int)y_shift;
     break;
 
   case Control_Segment_None:
     control->x_shift = 0;
     control->y_shift = 0;
     break;
+
+  case Control_Script_Feature:
+    /* the `glyph_idx' field holds the style; */
+    /* the `points' field holds the glyph index set */
+    break;
   }
 
-  control->ppems = number_set_reverse(ppem_set);
+  control->ppems = number_set_normalize(number_set_reverse(ppem_set));
   control->next = NULL;
 
   return control;
@@ -132,7 +140,7 @@ TA_control_free(Control* control)
 }
 
 
-sds
+static sds
 control_show_line(FONT* font,
                   Control* control)
 {
@@ -155,8 +163,9 @@ control_show_line(FONT* font,
 
   face = font->sfnts[control->font_idx].face;
   glyph_name_buf[0] = '\0';
-  if (FT_HAS_GLYPH_NAMES(face))
-    FT_Get_Glyph_Name(face, control->glyph_idx, glyph_name_buf, 64);
+  if (control->type != Control_Script_Feature
+      && FT_HAS_GLYPH_NAMES(face))
+    FT_Get_Glyph_Name(face, (FT_UInt)control->glyph_idx, glyph_name_buf, 64);
 
   points_buf = number_set_show(control->points, -1, -1);
   if (!points_buf)
@@ -225,6 +234,23 @@ control_show_line(FONT* font,
                        control->glyph_idx,
                        points_buf);
     break;
+
+  case Control_Script_Feature:
+    {
+      TA_StyleClass style_class = ta_style_classes[control->glyph_idx];
+      char feature_name[5];
+
+
+      feature_name[4] = '\0';
+      hb_tag_to_string(feature_tags[style_class->coverage], feature_name);
+
+      s = sdscatprintf(s, "%ld %s %s @ %s",
+                       control->font_idx,
+                       script_names[style_class->script],
+                       feature_name,
+                       points_buf);
+    }
+    break;
   }
 
 Exit:
@@ -363,7 +389,7 @@ Fail:
           break;
         p_end++;
       }
-      *errline_p = strndup(p_start, p_end - p_start);
+      *errline_p = strndup(p_start, (size_t)(p_end - p_start));
 
       /* construct data for `error_string_p' */
       if (context.error == TA_Err_Control_Invalid_Font_Index)
@@ -401,7 +427,7 @@ Fail:
       else
         *errpos_p = NULL;
 
-      *errlinenum_p = context.errline_num;
+      *errlinenum_p = (unsigned int)context.errline_num;
     }
   }
   else
@@ -411,6 +437,50 @@ Fail:
 }
 
 
+/* apply coverage data from the control instructions file */
+
+void
+TA_control_apply_coverage(SFNT* sfnt,
+                          FONT* font)
+{
+  Control* control = font->control;
+  TA_FaceGlobals globals = (TA_FaceGlobals)sfnt->face->autohint.data;
+  FT_UShort* gstyles = globals->glyph_styles;
+
+
+  while (control)
+  {
+    number_set_iter glyph_idx_iter;
+    int glyph_idx;
+    int style;
+
+
+    if (control->type != Control_Script_Feature)
+      goto Skip;
+    if (control->font_idx != sfnt->face->face_index)
+      goto Skip;
+
+    /* `control->glyph_idx' holds the style; */
+    /* `control->points' holds the glyph index set */
+    style = control->glyph_idx;
+    glyph_idx_iter.range = control->points;
+    glyph_idx = number_set_get_first(&glyph_idx_iter);
+
+    while (glyph_idx_iter.range)
+    {
+      /* assign new style but retain digit property */
+      gstyles[glyph_idx] &= TA_DIGIT;
+      gstyles[glyph_idx] |= style;
+
+      glyph_idx = number_set_get_next(&glyph_idx_iter);
+    }
+
+  Skip:
+    control = control->next;
+  }
+}
+
+
 /* node structure for control instruction data */
 
 typedef struct Node Node;
@@ -527,6 +597,13 @@ TA_control_build_tree(FONT* font)
     int ppem;
 
 
+    /* we don't store style information in the tree */
+    if (type == Control_Script_Feature)
+    {
+      control = control->next;
+      continue;
+    }
+
     ppems_iter.range = control->ppems;
     ppem = number_set_get_first(&ppems_iter);
 
