@@ -1,7 +1,7 @@
 /* taerror.c */
 
 /*
- * Copyright (C) 2011-2014 by Werner Lemberg.
+ * Copyright (C) 2011-2015 by Werner Lemberg.
  *
  * This file is part of the ttfautohint library, and may only be used,
  * modified, and distributed under the terms given in `COPYING'.  By
@@ -25,7 +25,7 @@ typedef const struct TA_error_
   const char* err_msg;
 } TA_error;
 
-TA_error TA_Errors[] =
+static TA_error TA_Errors[] =
 
 #undef __FTERRORS_H__
 #define FT_ERRORDEF(e, v, s) { e, s },
