@@ -93,20 +93,24 @@ TA_GlyphLoader_CheckPoints(TA_GlyphLoader loader,
 
 
 #define TA_GLYPHLOADER_CHECK_P(_loader, _count) \
-   ((_count) == 0 || ((_loader)->base.outline.n_points \
-                      + (_loader)->current.outline.n_points \
-                      + (unsigned long)(_count)) <= (_loader)->max_points)
+   ((_count) == 0 \
+    || ((FT_UInt)(_loader)->base.outline.n_points \
+        + (FT_UInt)(_loader)->current.outline.n_points \
+        + (FT_UInt)(_count)) <= (_loader)->max_points)
 
 #define TA_GLYPHLOADER_CHECK_C(_loader, _count) \
-   ((_count) == 0 || ((_loader)->base.outline.n_contours \
-                      + (_loader)->current.outline.n_contours \
-                      + (unsigned long)(_count)) <= (_loader)->max_contours)
+   ((_count) == 0 \
+    || ((FT_UInt)(_loader)->base.outline.n_contours \
+        + (FT_UInt)(_loader)->current.outline.n_contours \
+        + (FT_UInt)(_count)) <= (_loader)->max_contours)
 
 #define TA_GLYPHLOADER_CHECK_POINTS(_loader, _points, _contours) \
    ((TA_GLYPHLOADER_CHECK_P(_loader, _points) \
      && TA_GLYPHLOADER_CHECK_C(_loader, _contours)) \
     ? 0 \
-    : TA_GlyphLoader_CheckPoints((_loader), (_points), (_contours)))
+    : TA_GlyphLoader_CheckPoints((_loader), \
+                                 (FT_UInt)(_points), \
+                                 (FT_UInt)(_contours)))
 
 
 /* check that there is enough space to add */
