@@ -1,7 +1,7 @@
 /* taglobal.c */
 
 /*
- * Copyright (C) 2011-2014 by Werner Lemberg.
+ * Copyright (C) 2011-2015 by Werner Lemberg.
  *
  * This file is part of the ttfautohint library, and may only be used,
  * modified, and distributed under the terms given in `COPYING'.  By
@@ -21,7 +21,6 @@
 
 #include "taglobal.h"
 #include "taranges.h"
-#include "taharfbuzz.h"
 
 
 #undef SCRIPT
@@ -30,6 +29,7 @@
           { \
             TA_SCRIPT_ ## S, \
             ta_ ## s ## _uniranges, \
+            ta_ ## s ## _nonbase_uniranges, \
             sc1, sc2, sc3 \
           };
 
@@ -115,8 +115,8 @@ const char* ta_style_names[] =
 static FT_Error
 ta_face_globals_scan_composite(FT_Face face,
                                FT_Long gindex,
-                               FT_Byte gstyle,
-                               FT_Byte* gstyles,
+                               FT_UShort gstyle,
+                               FT_UShort* gstyles,
                                FT_Int nesting_level)
 {
   FT_Error error;
@@ -138,7 +138,7 @@ ta_face_globals_scan_composite(FT_Face face,
   if (nesting_level > 100)
     return FT_Err_Invalid_Table;
 
-  error = FT_Load_Glyph(face, gindex, FT_LOAD_NO_RECURSE);
+  error = FT_Load_Glyph(face, (FT_UInt)gindex, FT_LOAD_NO_RECURSE);
   if (error)
     return error;
 
@@ -170,7 +170,7 @@ ta_face_globals_scan_composite(FT_Face face,
                                &p_transform);
 
     if (p_index >= face->num_glyphs
-        || (gstyles[p_index] & ~TA_DIGIT) != TA_STYLE_UNASSIGNED)
+        || (gstyles[p_index] & TA_STYLE_MASK) != TA_STYLE_UNASSIGNED)
       continue;
 
     /* only take subglyphs that are not shifted vertically; */
@@ -209,14 +209,15 @@ ta_face_globals_compute_style_coverage(TA_FaceGlobals globals)
   FT_Error error;
   FT_Face face = globals->face;
   FT_CharMap old_charmap = face->charmap;
-  FT_Byte* gstyles = globals->glyph_styles;
+  FT_UShort* gstyles = globals->glyph_styles;
   FT_UInt ss;
   FT_UInt i;
   FT_UInt dflt = ~0U; /* a non-valid value */
 
 
   /* the value TA_STYLE_UNASSIGNED means `uncovered glyph' */
-  memset(globals->glyph_styles, TA_STYLE_UNASSIGNED, globals->glyph_count);
+  for (i = 0; i < (unsigned int)globals->glyph_count; i++)
+    gstyles[i] = TA_STYLE_UNASSIGNED;
 
   error = FT_Select_Charmap(face, FT_ENCODING_UNICODE);
   if (error)
@@ -235,7 +236,7 @@ ta_face_globals_compute_style_coverage(TA_FaceGlobals globals)
     TA_Script_UniRange range;
 
 
-    if (script_class->script_uni_ranges == NULL)
+    if (script_class->script_uni_ranges->first == 0)
       continue;
 
     /* scan all Unicode points in the range and */
@@ -245,7 +246,9 @@ ta_face_globals_compute_style_coverage(TA_FaceGlobals globals)
       if ((FT_UInt)style_class->script == globals->font->default_script)
         dflt = ss;
 
-      for (range = script_class->script_uni_ranges; range->first != 0; range++)
+      for (range = script_class->script_uni_ranges;
+           range->first != 0;
+           range++)
       {
         FT_ULong charcode = range->first;
         FT_UInt gindex;
@@ -255,8 +258,8 @@ ta_face_globals_compute_style_coverage(TA_FaceGlobals globals)
 
         if (gindex != 0
             && gindex < (FT_ULong)globals->glyph_count
-            && gstyles[gindex] == TA_STYLE_UNASSIGNED)
-          gstyles[gindex] = (FT_Byte)ss;
+            && (gstyles[gindex] & TA_STYLE_MASK) == TA_STYLE_UNASSIGNED)
+          gstyles[gindex] = (FT_UShort)ss;
 
         for (;;)
         {
@@ -266,8 +269,37 @@ ta_face_globals_compute_style_coverage(TA_FaceGlobals globals)
             break;
 
           if (gindex < (FT_ULong)globals->glyph_count
-              && gstyles[gindex] == TA_STYLE_UNASSIGNED)
-            gstyles[gindex] = (FT_Byte)ss;
+              && (gstyles[gindex] & TA_STYLE_MASK) == TA_STYLE_UNASSIGNED)
+            gstyles[gindex] = (FT_UShort)ss;
+        }
+      }
+
+      /* do the same for the script's non-base characters */
+      for (range = script_class->script_uni_nonbase_ranges;
+           range->first != 0;
+           range++)
+      {
+        FT_ULong charcode = range->first;
+        FT_UInt gindex;
+
+
+        gindex = FT_Get_Char_Index(face, charcode);
+
+        if (gindex != 0
+            && gindex < (FT_ULong)globals->glyph_count
+            && (gstyles[gindex] & TA_STYLE_MASK) == (FT_UShort)ss)
+          gstyles[gindex] |= TA_NONBASE;
+
+        for (;;)
+        {
+          charcode = FT_Get_Next_Char(face, charcode, &gindex);
+
+          if (gindex == 0 || charcode > range->last)
+            break;
+
+          if (gindex < (FT_ULong)globals->glyph_count
+              && (gstyles[gindex] & TA_STYLE_MASK) == (FT_UShort)ss)
+            gstyles[gindex] |= TA_NONBASE;
         }
       }
     }
@@ -311,7 +343,7 @@ ta_face_globals_compute_style_coverage(TA_FaceGlobals globals)
 
     for (nn = 0; nn < globals->glyph_count; nn++)
     {
-      if ((gstyles[nn] & ~TA_DIGIT) == TA_STYLE_UNASSIGNED)
+      if ((gstyles[nn] & TA_STYLE_MASK) == TA_STYLE_UNASSIGNED)
         continue;
 
       error = ta_face_globals_scan_composite(globals->face,
@@ -327,16 +359,16 @@ ta_face_globals_compute_style_coverage(TA_FaceGlobals globals)
 Exit:
   /* by default, all uncovered glyphs are set to the fallback style */
   /* XXX: Shouldn't we disable hinting or do something similar? */
-  if (globals->font->fallback_style != TA_STYLE_UNASSIGNED)
+  if (globals->font->fallback_style != (TA_Style)TA_STYLE_UNASSIGNED)
   {
     FT_Long nn;
 
 
     for (nn = 0; nn < globals->glyph_count; nn++)
     {
-      if ((gstyles[nn] & ~TA_DIGIT) == TA_STYLE_UNASSIGNED)
+      if ((gstyles[nn] & TA_STYLE_MASK) == TA_STYLE_UNASSIGNED)
       {
-        gstyles[nn] &= ~TA_STYLE_UNASSIGNED;
+        gstyles[nn] &= ~TA_STYLE_MASK;
         gstyles[nn] |= globals->font->fallback_style;
       }
     }
@@ -368,7 +400,7 @@ Exit:
 
     for (idx = 0; idx < globals->glyph_count; idx++)
     {
-      if ((gstyles[idx] & ~TA_DIGIT) == style_class->style)
+      if ((gstyles[idx] & TA_STYLE_MASK) == style_class->style)
       {
         if (!(count % 10))
           TA_LOG_GLOBAL((" "));
@@ -403,8 +435,11 @@ ta_face_globals_new(FT_Face face,
   TA_FaceGlobals globals;
 
 
-  globals = (TA_FaceGlobals)calloc(1, sizeof (TA_FaceGlobalsRec) +
-                                      face->num_glyphs * sizeof (FT_Byte));
+  /* we allocate an TA_FaceGlobals structure together */
+  /* with the glyph_styles array */
+  globals = (TA_FaceGlobals)calloc(
+              1, sizeof (TA_FaceGlobalsRec) +
+                 (FT_ULong)face->num_glyphs * sizeof (FT_UShort));
   if (!globals)
   {
     error = FT_Err_Out_Of_Memory;
@@ -413,7 +448,8 @@ ta_face_globals_new(FT_Face face,
 
   globals->face = face;
   globals->glyph_count = face->num_glyphs;
-  globals->glyph_styles = (FT_Byte*)(globals + 1);
+  /* right after the globals structure come the glyph styles */
+  globals->glyph_styles = (FT_UShort*)(globals + 1);
   globals->font = font;
   globals->hb_font = hb_ft_font_create(face, NULL);
 
