@@ -42,25 +42,25 @@
 /* load coverage tags */
 #undef COVERAGE
 #define COVERAGE(name, NAME, description, \
-                 tag1, tag2, tag3, tag4) \
+                 tag, tag1, tag2, tag3, tag4) \
           static const hb_tag_t name ## _coverage[] = \
           { \
             HB_TAG(tag1, tag2, tag3, tag4), \
             HB_TAG_NONE \
           };
 
-#include "tacover.h"
+#include "ttfautohint-coverages.h"
 
 
 /* define mapping between coverage tags and TA_Coverage */
 #undef COVERAGE
 #define COVERAGE(name, NAME, description, \
-                 tag1, tag2, tag3, tag4) \
+                 tag, tag1, tag2, tag3, tag4) \
           name ## _coverage,
 
 static const hb_tag_t* coverages[] =
 {
-#include "tacover.h"
+#include "ttfautohint-coverages.h"
 
   NULL /* TA_COVERAGE_DEFAULT */
 };
@@ -79,7 +79,7 @@ static const hb_script_t scripts[] =
 FT_Error
 ta_get_coverage(TA_FaceGlobals globals,
                 TA_StyleClass style_class,
-                FT_Byte* gstyles)
+                FT_UShort* gstyles)
 {
   hb_face_t* face;
 
@@ -168,7 +168,7 @@ ta_get_coverage(TA_FaceGlobals globals,
   count = 0;
 #endif
 
-  for (idx = -1; hb_set_next(gsub_lookups, &idx);)
+  for (idx = HB_SET_VALUE_INVALID; hb_set_next(gsub_lookups, &idx);)
   {
 #ifdef TA_DEBUG
     TA_LOG_GLOBAL((" %d", idx));
@@ -199,7 +199,7 @@ ta_get_coverage(TA_FaceGlobals globals,
   count = 0;
 #endif
 
-  for (idx = -1; hb_set_next(gpos_lookups, &idx);)
+  for (idx = HB_SET_VALUE_INVALID; hb_set_next(gpos_lookups, &idx);)
   {
 #ifdef TA_DEBUG
     TA_LOG_GLOBAL((" %d", idx));
@@ -247,7 +247,7 @@ ta_get_coverage(TA_FaceGlobals globals,
 
         GET_UTF8_CHAR(ch, p);
 
-        for (idx = -1; hb_set_next(gsub_lookups, &idx);)
+        for (idx = HB_SET_VALUE_INVALID; hb_set_next(gsub_lookups, &idx);)
         {
           hb_codepoint_t gidx = FT_Get_Char_Index(globals->face, ch);
 
@@ -323,7 +323,7 @@ ta_get_coverage(TA_FaceGlobals globals,
   count = 0;
 #endif
 
-  for (idx = -1; hb_set_next(gsub_glyphs, &idx);)
+  for (idx = HB_SET_VALUE_INVALID; hb_set_next(gsub_glyphs, &idx);)
   {
 #ifdef TA_DEBUG
     if (!(count % 10))
@@ -341,7 +341,7 @@ ta_get_coverage(TA_FaceGlobals globals,
       continue;
 
     if (gstyles[idx] == TA_STYLE_UNASSIGNED)
-      gstyles[idx] = (FT_Byte)style_class->style;
+      gstyles[idx] = (FT_UShort)style_class->style;
 #ifdef TA_DEBUG
     else
       TA_LOG_GLOBAL(("*"));
@@ -368,7 +368,7 @@ Exit:
 /* construct HarfBuzz features */
 #undef COVERAGE
 #define COVERAGE(name, NAME, description, \
-                 tag1, tag2, tag3, tag4) \
+                 tag, tag1, tag2, tag3, tag4) \
           static const hb_feature_t name ## _feature[] = \
           { \
             { \
@@ -377,18 +377,18 @@ Exit:
             } \
           };
 
-#include "tacover.h"
+#include "ttfautohint-coverages.h"
 
 
 /* define mapping between HarfBuzz features and TA_Coverage */
 #undef COVERAGE
 #define COVERAGE(name, NAME, description, \
-                 tag1, tag2, tag3, tag4) \
+                 tag, tag1, tag2, tag3, tag4) \
           name ## _feature,
 
 static const hb_feature_t* features[] =
 {
-#include "tacover.h"
+#include "ttfautohint-coverages.h"
 
   NULL /* TA_COVERAGE_DEFAULT */
 };
@@ -418,7 +418,7 @@ ta_get_char_index(TA_StyleMetrics metrics,
 
   if (feature)
   {
-    FT_UInt upem = metrics->globals->face->units_per_EM;
+    int upem = (int)metrics->globals->face->units_per_EM;
 
     hb_font_t* font = metrics->globals->hb_font;
     hb_buffer_t* buf = hb_buffer_create();
