@@ -31,10 +31,15 @@
 #include "ta.h"
 
 
+/* in file `tafeature.c' */
+extern const hb_tag_t feature_tags[];
+extern size_t feature_tags_size;
+
+
 FT_Error
 ta_get_coverage(TA_FaceGlobals globals,
                 TA_StyleClass style_class,
-                FT_Byte* gstyles);
+                FT_UShort* gstyles);
 
 FT_Error
 ta_get_char_index(TA_StyleMetrics metrics,
