@@ -1,7 +1,7 @@
 /* tahmtx.c */
 
 /*
- * Copyright (C) 2011-2013 by Werner Lemberg.
+ * Copyright (C) 2011-2015 by Werner Lemberg.
  *
  * This file is part of the ttfautohint library, and may only be used,
  * modified, and distributed under the terms given in `COPYING'.  By
@@ -41,7 +41,7 @@ TA_sfnt_update_hmtx_table(SFNT* sfnt,
 
   hmtx_table->len += 2;
   /* make the allocated buffer length a multiple of 4 */
-  buf_len = (hmtx_table->len + 3) & ~3;
+  buf_len = (hmtx_table->len + 3) & ~3U;
   buf_new = (FT_Byte*)realloc(hmtx_table->buf, buf_len);
   if (!buf_new)
   {
