@@ -1,7 +1,7 @@
 /* talatin.c */
 
 /*
- * Copyright (C) 2011-2014 by Werner Lemberg.
+ * Copyright (C) 2011-2015 by Werner Lemberg.
  *
  * This file is part of the ttfautohint library, and may only be used,
  * modified, and distributed under the terms given in `COPYING'.  By
@@ -35,6 +35,10 @@
 #include <numberset.h>
 
 
+/* needed for computation of round vs. flat segments */
+#define FLAT_THRESHOLD(x)  (x / 14)
+
+
 /* find segments and links, compute all stem widths, and initialize */
 /* standard width and height for the glyph with given charcode */
 
@@ -259,8 +263,8 @@ ta_latin_metrics_init_blues(TA_LatinMetrics metrics,
 {
   FT_Pos flats[TA_BLUE_STRING_MAX_LEN];
   FT_Pos rounds[TA_BLUE_STRING_MAX_LEN];
-  FT_Int num_flats;
-  FT_Int num_rounds;
+  FT_UInt num_flats;
+  FT_UInt num_rounds;
 
   TA_LatinBlue blue;
   FT_Error error;
@@ -272,6 +276,8 @@ ta_latin_metrics_init_blues(TA_LatinMetrics metrics,
   TA_Blue_Stringset bss = sc->blue_stringset;
   const TA_Blue_StringRec* bs = &ta_blue_stringsets[bss];
 
+  FT_Pos flat_threshold = FLAT_THRESHOLD(metrics->units_per_em);
+
 
   /* we walk over the blue character strings as specified in the  */
   /* style's entry in the `ta_blue_stringset' array */
@@ -360,9 +366,10 @@ ta_latin_metrics_init_blues(TA_LatinMetrics metrics,
 
       error = FT_Load_Glyph(face, glyph_index, FT_LOAD_NO_SCALE);
       outline = face->glyph->outline;
-      if (error || outline.n_points <= 0)
+      /* reject glyphs that don't produce any rendering */
+      if (error || outline.n_points <= 2)
       {
-        TA_LOG_GLOBAL(("  U+%04lX contains no outlines\n", ch));
+        TA_LOG_GLOBAL(("  U+%04lX contains no (usable) outlines\n", ch));
         continue;
       }
 
@@ -689,7 +696,7 @@ ta_latin_metrics_init_blues(TA_LatinMetrics metrics,
          * now set the `round' flag depending on the segment's kind:
          *
          * - if the horizontal distance between the first and last
-         *   `on' point is larger than upem/8 (value 8 is heuristic)
+         *   `on' point is larger than a heuristic threshold
          *   we have a flat segment
          * - if either the first or the last point of the segment is
          *   an `off' point, the segment is round, otherwise it is
@@ -697,9 +704,9 @@ ta_latin_metrics_init_blues(TA_LatinMetrics metrics,
          */
         if (best_on_point_first >= 0
             && best_on_point_last >= 0
-            && (FT_UInt)(TA_ABS(points[best_on_point_last].x
+            && (TA_ABS(points[best_on_point_last].x
                                 - points[best_on_point_first].x))
-                 > metrics->units_per_em / 8)
+                 > flat_threshold)
           round = 0;
         else
           round = FT_BOOL(FT_CURVE_TAG(outline.tags[best_segment_first])
@@ -964,7 +971,7 @@ ta_latin_metrics_scale_dim(TA_LatinMetrics metrics,
   /* (if we do x-height snapping for this ppem value) */
   if (!number_set_is_element(
         metrics->root.globals->font->x_height_snapping_exceptions,
-        ppem))
+        (int)ppem))
   {
     TA_LatinAxis Axis = &metrics->axis[TA_DIMENSION_VERT];
     TA_LatinBlue blue = NULL;
@@ -1069,8 +1076,11 @@ ta_latin_metrics_scale_dim(TA_LatinMetrics metrics,
 
   if (dim == TA_DIMENSION_VERT)
   {
-    TA_LOG_GLOBAL(("blue zones (style `%s')\n",
-                   ta_style_names[metrics->root.style_class->style]));
+#ifdef TA_DEBUG
+    if (axis->blue_count)
+      TA_LOG_GLOBAL(("blue zones (style `%s')\n",
+                     ta_style_names[metrics->root.style_class->style]));
+#endif
 
     /* scale the blue zones */
     for (nn = 0; nn < axis->blue_count; nn++)
@@ -1219,6 +1229,7 @@ FT_Error
 ta_latin_hints_compute_segments(TA_GlyphHints hints,
                                 TA_Dimension dim)
 {
+  TA_LatinMetrics metrics = (TA_LatinMetrics)hints->metrics;
   TA_AxisHints axis = &hints->axis[dim];
   FT_Error error = FT_Err_Ok;
 
@@ -1229,6 +1240,8 @@ ta_latin_hints_compute_segments(TA_GlyphHints hints,
   TA_Point* contour_limit = contour + hints->num_contours;
   TA_Direction major_dir, segment_dir;
 
+  FT_Pos flat_threshold = FLAT_THRESHOLD(metrics->units_per_em);
+
 
   memset(&seg0, 0, sizeof (TA_SegmentRec));
   seg0.score = 32000;
@@ -1275,6 +1288,8 @@ ta_latin_hints_compute_segments(TA_GlyphHints hints,
 
     FT_Pos min_pos = 32000; /* minimum segment pos != min_coord */
     FT_Pos max_pos = -32000; /* maximum segment pos != max_coord */
+    FT_Pos min_on_pos = 32000;
+    FT_Pos max_on_pos = -32000;
     FT_Bool passed;
 
 
@@ -1316,6 +1331,16 @@ ta_latin_hints_compute_segments(TA_GlyphHints hints,
         if (u > max_pos)
           max_pos = u;
 
+        /* get minimum and maximum coordinate of on points */
+        if (!(point->flags & TA_FLAG_CONTROL))
+        {
+          v = point->v;
+          if (v < min_on_pos)
+            min_on_pos = v;
+          if (v > max_on_pos)
+            max_on_pos = v;
+        }
+
         if (point->out_dir != segment_dir
             || point == last)
         {
@@ -1324,8 +1349,10 @@ ta_latin_hints_compute_segments(TA_GlyphHints hints,
           segment->pos = (FT_Short)((min_pos + max_pos) >> 1);
 
           /* a segment is round if either its first or last point */
-          /* is a control point */
-          if ((segment->first->flags | point->flags) & TA_FLAG_CONTROL)
+          /* is a control point, and the length of the on points */
+          /* inbetween doesn't exceed a heuristic limit */
+          if ((segment->first->flags | point->flags) & TA_FLAG_CONTROL
+              && (max_on_pos - min_on_pos) < flat_threshold)
             segment->flags |= TA_EDGE_ROUND;
 
           /* compute segment size */
@@ -1370,10 +1397,19 @@ ta_latin_hints_compute_segments(TA_GlyphHints hints,
         segment[0] = seg0;
 
         segment->dir = (FT_Char)segment_dir;
-        min_pos = max_pos = point->u;
         segment->first = point;
         segment->last = point;
 
+        min_pos = max_pos = point->u;
+
+        if (point->flags & TA_FLAG_CONTROL)
+        {
+          min_on_pos = 32000;
+          max_on_pos = -32000;
+        }
+        else
+          min_on_pos = max_on_pos = point->v;
+
         on_edge = 1;
 
         if (point->out_dir != point->next->in_dir)
@@ -1878,7 +1914,7 @@ ta_latin_hints_compute_edges(TA_GlyphHints hints,
       /* example: the `c' in cour.pfa at size 13 */
 
       if (edge->serif && edge->link)
-        edge->serif = 0;
+        edge->serif = NULL;
     }
   }
 
@@ -1912,7 +1948,7 @@ ta_latin_hints_detect_features(TA_GlyphHints hints,
 
 /* compute all edges which lie within blue zones */
 
-void
+static void
 ta_latin_hints_compute_blue_edges(TA_GlyphHints hints,
                                   TA_LatinMetrics metrics)
 {
@@ -2091,11 +2127,19 @@ ta_latin_hints_init(TA_GlyphHints hints,
     other_flags |= TA_LATIN_HINTS_MONO;
 
   /* in `light' hinting mode we disable horizontal hinting completely; */
-  /* we also do it if the face is italic */
+  /* we also do it if the face is italic -- */
+  /* however, if warping is enabled (which only works in `light' hinting */
+  /* mode), advance widths get adjusted, too */
   if (mode == FT_RENDER_MODE_LIGHT
       || (face->style_flags & FT_STYLE_FLAG_ITALIC) != 0)
     scaler_flags |= TA_SCALER_FLAG_NO_HORIZONTAL;
 
+#ifdef TA_CONFIG_OPTION_USE_WARPER
+  /* get (global) warper flag */
+  if (!metrics->root.globals->module->warping)
+    scaler_flags |= TA_SCALER_FLAG_NO_WARPER;
+#endif
+
   hints->scaler_flags = scaler_flags;
   hints->other_flags = other_flags;
 
@@ -2108,10 +2152,10 @@ ta_latin_hints_init(TA_GlyphHints hints,
 
 static FT_Pos
 ta_latin_snap_width(TA_Width widths,
-                    FT_Int count,
+                    FT_UInt count,
                     FT_Pos width)
 {
-  int n;
+  FT_UInt n;
   FT_Pos best = 64 + 32 + 2;
   FT_Pos reference = width;
   FT_Pos scaled;
@@ -2360,7 +2404,7 @@ ta_latin_align_serif_edge(TA_GlyphHints hints,
 
 /* the main grid-fitting routine */
 
-void
+static void
 ta_latin_hint_edges(TA_GlyphHints hints,
                     TA_Dimension dim)
 {
@@ -2414,7 +2458,7 @@ ta_latin_hint_edges(TA_GlyphHints hints,
         FT_Byte neutral2 = edge2->flags & TA_EDGE_NEUTRAL;
 
 
-        if ((neutral && neutral2) || neutral2)
+        if (neutral2)
         {
           edge2->blue_edge = NULL;
           edge2->flags &= ~TA_EDGE_NEUTRAL;
@@ -2960,7 +3004,8 @@ ta_latin_hint_edges(TA_GlyphHints hints,
 /* apply the complete hinting algorithm to a latin glyph */
 
 static FT_Error
-ta_latin_hints_apply(TA_GlyphHints hints,
+ta_latin_hints_apply(FT_UInt glyph_index,
+                     TA_GlyphHints hints,
                      FT_Outline* outline,
                      TA_LatinMetrics metrics)
 {
@@ -2976,7 +3021,8 @@ ta_latin_hints_apply(TA_GlyphHints hints,
 
   /* analyze glyph outline */
 #ifdef TA_CONFIG_OPTION_USE_WARPER
-  if (metrics->root.scaler.render_mode == FT_RENDER_MODE_LIGHT
+  if ((metrics->root.scaler.render_mode == FT_RENDER_MODE_LIGHT
+       && TA_HINTS_DO_WARP(hints))
       || TA_HINTS_DO_HORIZONTAL(hints))
 #else
   if (TA_HINTS_DO_HORIZONTAL(hints))
@@ -3001,7 +3047,9 @@ ta_latin_hints_apply(TA_GlyphHints hints,
     if (error)
       goto Exit;
 
-    ta_latin_hints_compute_blue_edges(hints, metrics);
+    /* apply blue zones to base characters only */
+    if (!(metrics->root.globals->glyph_styles[glyph_index] & TA_NONBASE))
+      ta_latin_hints_compute_blue_edges(hints, metrics);
   }
 
   /* grid-fit the outline */
@@ -3009,7 +3057,8 @@ ta_latin_hints_apply(TA_GlyphHints hints,
   {
 #ifdef TA_CONFIG_OPTION_USE_WARPER
     if (dim == TA_DIMENSION_HORZ
-        && metrics->root.scaler.render_mode == FT_RENDER_MODE_LIGHT)
+        && metrics->root.scaler.render_mode == FT_RENDER_MODE_LIGHT
+        && TA_HINTS_DO_WARP(hints))
     {
       TA_WarperRec warper;
       FT_Fixed scale;
@@ -3021,7 +3070,7 @@ ta_latin_hints_apply(TA_GlyphHints hints,
 
       continue;
     }
-#endif
+#endif /* TA_CONFIG_OPTION_USE_WARPER */
 
     if ((dim == TA_DIMENSION_HORZ && TA_HINTS_DO_HORIZONTAL(hints))
         || (dim == TA_DIMENSION_VERT && TA_HINTS_DO_VERTICAL(hints)))
