@@ -1,7 +1,7 @@
 /* taloader.c */
 
 /*
- * Copyright (C) 2011-2014 by Werner Lemberg.
+ * Copyright (C) 2011-2015 by Werner Lemberg.
  *
  * This file is part of the ttfautohint library, and may only be used,
  * modified, and distributed under the terms given in `COPYING'.  By
@@ -145,8 +145,8 @@ ta_loader_load_g(TA_Loader loader,
     loader->trans_delta = internal->glyph_delta;
 
     inverse = loader->trans_matrix;
-    FT_Matrix_Invert(&inverse);
-    FT_Vector_Transform(&loader->trans_delta, &inverse);
+    if (!FT_Matrix_Invert(&inverse))
+      FT_Vector_Transform(&loader->trans_delta, &inverse);
   }
 #endif
 
@@ -169,13 +169,13 @@ ta_loader_load_g(TA_Loader loader,
 
     memcpy(gloader->current.outline.points,
            slot->outline.points,
-           slot->outline.n_points * sizeof (FT_Vector));
+           (size_t)slot->outline.n_points * sizeof (FT_Vector));
     memcpy(gloader->current.outline.contours,
            slot->outline.contours,
-           slot->outline.n_contours * sizeof (short));
+           (size_t)slot->outline.n_contours * sizeof (short));
     memcpy(gloader->current.outline.tags,
            slot->outline.tags,
-           slot->outline.n_points * sizeof (char));
+           (size_t)slot->outline.n_points * sizeof (char));
 
     gloader->current.outline.n_points = slot->outline.n_points;
     gloader->current.outline.n_contours = slot->outline.n_contours;
@@ -201,7 +201,8 @@ ta_loader_load_g(TA_Loader loader,
 
 
       if (writing_system_class->style_hints_apply)
-        writing_system_class->style_hints_apply(hints,
+        writing_system_class->style_hints_apply(glyph_index,
+                                                hints,
                                                 &gloader->current.outline,
                                                 metrics);
     }
@@ -286,7 +287,7 @@ ta_loader_load_g(TA_Loader loader,
       TA_SubGlyph subglyph;
 
 
-      start_point = gloader->base.outline.n_points;
+      start_point = (FT_UInt)gloader->base.outline.n_points;
 
       /* first of all, copy the subglyph descriptors in the glyph loader */
       error = TA_GlyphLoader_CheckSubGlyphs(gloader, num_subglyphs);
@@ -316,9 +317,9 @@ ta_loader_load_g(TA_Loader loader,
         pp1 = loader->pp1;
         pp2 = loader->pp2;
 
-        num_base_points = gloader->base.outline.n_points;
+        num_base_points = (FT_UInt)gloader->base.outline.n_points;
 
-        error = ta_loader_load_g(loader, scaler, subglyph->index,
+        error = ta_loader_load_g(loader, scaler, (FT_UInt)subglyph->index,
                                  load_flags, depth + 1);
         if (error)
           goto Exit;
@@ -332,7 +333,7 @@ ta_loader_load_g(TA_Loader loader,
           loader->pp2 = pp2;
         }
 
-        num_points = gloader->base.outline.n_points;
+        num_points = (FT_UInt)gloader->base.outline.n_points;
         num_new_points = num_points - num_base_points;
 
         /* now perform the transformation required for this subglyph */
@@ -351,14 +352,14 @@ ta_loader_load_g(TA_Loader loader,
         /* apply offset */
         if (!(subglyph->flags & FT_SUBGLYPH_FLAG_ARGS_ARE_XY_VALUES))
         {
-          FT_Int k = subglyph->arg1;
-          FT_UInt l = subglyph->arg2;
+          FT_UInt k = (FT_UInt)subglyph->arg1;
+          FT_UInt l = (FT_UInt)subglyph->arg2;
           FT_Vector* p1;
           FT_Vector* p2;
 
 
           if (start_point + k >= num_base_points
-              || l >= (FT_UInt)num_new_points)
+              || l >= num_new_points)
           {
             error = FT_Err_Invalid_Composite;
             goto Exit;
@@ -507,7 +508,7 @@ ta_loader_load_glyph(FONT* font,
 
 
   if (!size)
-    return FT_Err_Invalid_Argument;
+    return FT_Err_Invalid_Size_Handle;
 
   memset(&scaler, 0, sizeof (TA_ScalerRec));
 
