@@ -62,12 +62,9 @@ parse_name_header(FT_Byte** curp,
 
   p = *curp;
 
-  n->format = *(p++) << 8;
-  n->format += *(p++);
-  n->name_count = *(p++) << 8;
-  n->name_count += *(p++);
-  n->string_offset = *(p++) << 8;
-  n->string_offset += *(p++);
+  n->format = NEXT_USHORT(p);
+  n->name_count = NEXT_USHORT(p);
+  n->string_offset = NEXT_USHORT(p);
 
   n->name_records = NULL;
   n->lang_tag_records = NULL;
@@ -86,8 +83,7 @@ parse_name_header(FT_Byte** curp,
     if (q + 2 > endp)
       return FT_Err_Invalid_Table;
 
-    n->lang_tag_count = *(q++) << 8;
-    n->lang_tag_count += *q;
+    n->lang_tag_count = NEXT_USHORT(q);
 
     startp += 2 + 4 * n->lang_tag_count;
   }
@@ -139,20 +135,14 @@ parse_name_records(FT_Byte** curp,
     FT_UShort l;
 
 
-    r->platform_id = *(p++) << 8;
-    r->platform_id += *(p++);
-    r->encoding_id = *(p++) << 8;
-    r->encoding_id += *(p++);
-    r->language_id = *(p++) << 8;
-    r->language_id += *(p++);
-    r->name_id = *(p++) << 8;
-    r->name_id += *(p++);
+    r->platform_id = NEXT_USHORT(p);
+    r->encoding_id = NEXT_USHORT(p);
+    r->language_id = NEXT_USHORT(p);
+    r->name_id = NEXT_USHORT(p);
 
-    r->len = *(p++) << 8;
-    r->len += *(p++);
+    r->len = NEXT_USHORT(p);
 
-    offset = *(p++) << 8;
-    offset += *(p++);
+    offset = NEXT_USHORT(p);
 
     s = buf + n->string_offset + offset;
 
@@ -183,7 +173,7 @@ parse_name_records(FT_Byte** curp,
       /* (two-byte) UTF-16BE for everything else */
 
       /* we need an even number of bytes */
-      r->len &= ~1;
+      r->len &= ~1U;
 
       /* ignore entries which contain only NULL bytes */
       for (l = 0; l < r->len; l++)
@@ -227,7 +217,7 @@ parse_name_records(FT_Byte** curp,
 }
 
 
-FT_Error
+static FT_Error
 parse_lang_tag_records(FT_Byte** curp,
                        Naming_Table* n,
                        FT_Byte* buf,
@@ -258,11 +248,9 @@ parse_lang_tag_records(FT_Byte** curp,
     FT_Byte* s;
 
 
-    r->len = *(p++) << 8;
-    r->len += *(p++);
+    r->len = NEXT_USHORT(p);
 
-    offset = *(p++) << 8;
-    offset += *(p++);
+    offset = NEXT_USHORT(p);
 
     s = buf + n->string_offset + offset;
 
@@ -321,7 +309,7 @@ build_name_table(Naming_Table* n,
   /* `offset to the start of string storage (from start of table)', */
   /* but this isn't enforced by the major rendering engines */
   /* as long as the final string offsets are valid */
-  string_offset = (buf_new_len <= 0xFFFF) ? buf_new_len : 0xFFFF;
+  string_offset = (buf_new_len <= 0xFFFF) ? (FT_UShort)buf_new_len : 0xFFFF;
 
   p = buf_new;
 
@@ -390,7 +378,7 @@ build_name_table(Naming_Table* n,
   /* reallocate the buffer to fit its real size */
   buf_new_len += data_len;
   /* make the allocated buffer length a multiple of 4 */
-  len = (buf_new_len + 3) & ~3;
+  len = (buf_new_len + 3) & ~3U;
 
   buf_new_resized = (FT_Byte*)realloc(buf_new, len);
   if (!buf_new_resized)
