@@ -1,7 +1,7 @@
 /* tapost.c */
 
 /*
- * Copyright (C) 2011-2014 by Werner Lemberg.
+ * Copyright (C) 2011-2015 by Werner Lemberg.
  *
  * This file is part of the ttfautohint library, and may only be used,
  * modified, and distributed under the terms given in `COPYING'.  By
@@ -41,10 +41,10 @@ TA_sfnt_update_post_table(SFNT* sfnt,
   if (post_table->processed)
     return TA_Err_Ok;
 
-  version = buf[0] << 24;
-  version += buf[1] << 16;
-  version += buf[2] << 8;
-  version += buf[3];
+  version = (FT_ULong)(buf[0] << 24
+                       | buf[1] << 16
+                       | buf[2] << 8
+                       | buf[3]);
 
   /* since we have to add a non-standard glyph name, */
   /* we must convert `name' tables in formats 1.0 and 2.5 into format 2.0 */
@@ -63,8 +63,7 @@ TA_sfnt_update_post_table(SFNT* sfnt,
     FT_Byte* p_new;
 
 
-    num_glyphs = buf[32] << 8;
-    num_glyphs += buf[33];
+    num_glyphs = (FT_UShort)(buf[32] << 8 | buf[33]);
 
     p = buf + 34;
     max_name_idx = 0;
@@ -75,8 +74,7 @@ TA_sfnt_update_post_table(SFNT* sfnt,
       FT_UShort idx;
 
 
-      idx = *(p++) << 8;
-      idx += *(p++);
+      idx = NEXT_USHORT(p);
       if (idx >= 258)
       {
         idx -= 257;
@@ -90,7 +88,7 @@ TA_sfnt_update_post_table(SFNT* sfnt,
     buf_new_len = post_table->len + 2 + TTFAUTOHINT_GLYPH_LEN;
 
     /* make the allocated buffer length a multiple of 4 */
-    len = (buf_new_len + 3) & ~3;
+    len = (buf_new_len + 3) & ~3U;
     buf_new = (FT_Byte*)malloc(len);
     if (!buf_new)
       return FT_Err_Out_Of_Memory;
@@ -121,7 +119,7 @@ TA_sfnt_update_post_table(SFNT* sfnt,
     *(p_new + 1) = LOW(max_name_idx + 1 + 257);
     p_new += 2;
 
-    memcpy(p_new, p, buf + buf_len - p); /* names */
+    memcpy(p_new, p, (size_t)(buf + buf_len - p)); /* names */
     p_new += buf + buf_len - p;
 
     strncpy((char*)p_new, TTFAUTOHINT_GLYPH_FIRST_BYTE TTFAUTOHINT_GLYPH,
