@@ -30,7 +30,9 @@
  * coverage.
  *
  * Note that styles using `TA_COVERAGE_DEFAULT' should always
- * come after styles with other coverages.
+ * come after styles with other coverages.  Also note that
+ * fallback scripts only use `AF_COVERAGE_DEFAULT' for its
+ * style.
  *
  * Example:
  *
@@ -54,10 +56,10 @@
 #undef META_STYLE_LATIN
 #define META_STYLE_LATIN(s, S, ds) \
           STYLE_LATIN(s, S, c2cp, C2CP, ds, \
-                      "petite capticals from capitals", \
+                      "petite capitals from capitals", \
                       PETITE_CAPITALS_FROM_CAPITALS) \
           STYLE_LATIN(s, S, c2sc, C2SC, ds, \
-                      "small capticals from capitals", \
+                      "small capitals from capitals", \
                       SMALL_CAPITALS_FROM_CAPITALS) \
           STYLE_LATIN(s, S, ordn, ORDN, ds, \
                       "ordinals", \
@@ -84,6 +86,14 @@
                       "default", \
                       DEFAULT)
 
+
+STYLE(arab_dflt, ARAB_DFLT,
+      "Arabic default style",
+      TA_WRITING_SYSTEM_LATIN,
+      TA_SCRIPT_ARAB,
+      TA_BLUE_STRINGSET_ARAB,
+      TA_COVERAGE_DEFAULT)
+
 META_STYLE_LATIN(cyrl, CYRL, "Cyrillic")
 
 STYLE(deva_dflt, DEVA_DFLT,
@@ -102,8 +112,29 @@ STYLE(hebr_dflt, HEBR_DFLT,
       TA_BLUE_STRINGSET_HEBR,
       TA_COVERAGE_DEFAULT)
 
+STYLE(lao_dflt, LAO_DFLT,
+      "Lao default style",
+      TA_WRITING_SYSTEM_LATIN,
+      TA_SCRIPT_LAO,
+      TA_BLUE_STRINGSET_LAO,
+      TA_COVERAGE_DEFAULT)
+
 META_STYLE_LATIN(latn, LATN, "Latin")
 
+STYLE(latb_dflt, LATB_DFLT,
+      "Latin subscript fallback default style",
+      TA_WRITING_SYSTEM_LATIN,
+      TA_SCRIPT_LATB,
+      TA_BLUE_STRINGSET_LATB,
+      TA_COVERAGE_DEFAULT)
+
+STYLE(latp_dflt, LATP_DFLT,
+      "Latin superscript fallback default style",
+      TA_WRITING_SYSTEM_LATIN,
+      TA_SCRIPT_LATP,
+      TA_BLUE_STRINGSET_LATP,
+      TA_COVERAGE_DEFAULT)
+
 #ifdef FT_OPTION_AUTOFIT2
 STYLE(ltn2_dflt, LTN2_DFLT,
       "Latin 2 default style",
@@ -120,6 +151,13 @@ STYLE(telu_dflt, TELU_DFLT,
       TA_BLUE_STRINGSET_TELU,
       TA_COVERAGE_DEFAULT)
 
+STYLE(thai_dflt, THAI_DFLT,
+      "Thai default style",
+      TA_WRITING_SYSTEM_LATIN,
+      TA_SCRIPT_THAI,
+      TA_BLUE_STRINGSET_THAI,
+      TA_COVERAGE_DEFAULT)
+
 STYLE(none_dflt, NONE_DFLT,
       "no style",
       TA_WRITING_SYSTEM_DUMMY,
