@@ -1,7 +1,7 @@
 /* tatables.c */
 
 /*
- * Copyright (C) 2011-2014 by Werner Lemberg.
+ * Copyright (C) 2011-2015 by Werner Lemberg.
  *
  * This file is part of the ttfautohint library, and may only be used,
  * modified, and distributed under the terms given in `COPYING'.  By
@@ -53,12 +53,7 @@ TA_table_compute_checksum(FT_Byte* buf,
   /* we expect that the length of `buf' is a multiple of 4 */
 
   while (buf < end_buf)
-  {
-    checksum += *(buf++) << 24;
-    checksum += *(buf++) << 16;
-    checksum += *(buf++) << 8;
-    checksum += *(buf++);
-  }
+    checksum += NEXT_ULONG(buf);
 
   return checksum;
 }
@@ -167,7 +162,7 @@ TA_font_compute_table_offsets(FONT* font,
 
     /* table offsets must be multiples of 4; */
     /* this also fits the actual buffer lengths */
-    offset += (table->len + 3) & ~3;
+    offset += (table->len + 3) & ~3U;
   }
 }
 
