@@ -28,7 +28,7 @@ TA_get_current_time(FT_ULong* high,
   /* there have been 24107 days between January 1st, 1904 (the epoch of */
   /* OpenType), and January 1st, 1970 (the epoch of the `time' function) */
   TA_ULongLong seconds_to_1970 = 24107 * 24 * 60 * 60;
-  TA_ULongLong seconds_to_today = seconds_to_1970 + time(NULL);
+  TA_ULongLong seconds_to_today = seconds_to_1970 + (TA_ULongLong)time(NULL);
 
 
   *high = (FT_ULong)(seconds_to_today >> 32);
