@@ -1,7 +1,7 @@
 /* tattc.c */
 
 /*
- * Copyright (C) 2011-2014 by Werner Lemberg.
+ * Copyright (C) 2011-2015 by Werner Lemberg.
  *
  * This file is part of the ttfautohint library, and may only be used,
  * modified, and distributed under the terms given in `COPYING'.  By
@@ -37,7 +37,7 @@ TA_font_build_TTC_header(FONT* font,
   FT_Byte* p;
 
 
-  len = (font->have_DSIG ? 24 : 12) + 4 * num_sfnts;
+  len = (FT_ULong)((font->have_DSIG ? 24 : 12) + 4 * num_sfnts);
   buf = (FT_Byte*)malloc(len);
   if (!buf)
     return FT_Err_Out_Of_Memory;
@@ -198,11 +198,11 @@ TA_font_build_TTC(FONT* font)
   if (error)
     return error;
 
-  TTF_header_bufs = (FT_Byte**)calloc(1, num_sfnts * sizeof (FT_Byte*));
+  TTF_header_bufs = (FT_Byte**)calloc(1, (size_t)num_sfnts * sizeof (FT_Byte*));
   if (!TTF_header_bufs)
     goto Err;
 
-  TTF_header_lens = (FT_ULong*)malloc(num_sfnts * sizeof (FT_ULong));
+  TTF_header_lens = (FT_ULong*)malloc((size_t)num_sfnts * sizeof (FT_ULong));
   if (!TTF_header_lens)
     goto Err;
 
@@ -222,7 +222,7 @@ TA_font_build_TTC(FONT* font)
 
   /* get font length from last SFNT table array element */
   font->out_len = tables[num_tables - 1].offset
-                  + ((tables[num_tables - 1].len + 3) & ~3);
+                  + ((tables[num_tables - 1].len + 3) & ~3U);
   font->out_buf = (FT_Byte*)malloc(font->out_len);
   if (!font->out_buf)
   {
@@ -249,7 +249,7 @@ TA_font_build_TTC(FONT* font)
 
     /* buffer length is a multiple of 4 */
     memcpy(font->out_buf + table->offset,
-           table->buf, (table->len + 3) & ~3);
+           table->buf, (table->len + 3) & ~3U);
   }
 
   error = TA_Err_Ok;
