@@ -1,7 +1,7 @@
 /* tatypes.h */
 
 /*
- * Copyright (C) 2011-2014 by Werner Lemberg.
+ * Copyright (C) 2011-2015 by Werner Lemberg.
  *
  * This file is part of the ttfautohint library, and may only be used,
  * modified, and distributed under the terms given in `COPYING'.  By
@@ -90,12 +90,23 @@ extern void* _ta_debug_hints;
 
 #define TA_ABS(a) ((a) < 0 ? -(a) : (a))
 
+/* from file `ftconfig.h' (2015-03-02) from FreeType */
+/* typeof condition taken from gnulib's `intprops.h' header file */
+#if (__GNUC__ >= 2 \
+     ||  defined(__IBM__TYPEOF__) \
+     || (__SUNPRO_C >= 0x5110 && !__STDC__))
+#define TYPEOF(type) (__typeof__ (type))
+#else
+#define TYPEOF(type) /* empty */
+#endif
+
 /* from file `ftobjs.h' from FreeType */
-#define TA_PAD_FLOOR(x, n) ((x) & ~((n) - 1))
+/* we use the TYPEOF macro to suppress signedness compilation warnings */
+#define TA_PAD_FLOOR(x, n) ((x) & ~TYPEOF(x)((n) - 1))
 #define TA_PAD_ROUND(x, n) TA_PAD_FLOOR((x) + ((n) / 2), n)
 #define TA_PAD_CEIL(x, n) TA_PAD_FLOOR((x) + ((n) - 1), n)
 
-#define TA_PIX_FLOOR(x) ((x) & ~63)
+#define TA_PIX_FLOOR(x) ((x) & ~TYPEOF(x)63)
 #define TA_PIX_ROUND(x) TA_PIX_FLOOR((x) + 32)
 #define TA_PIX_CEIL(x) TA_PIX_FLOOR((x) + 63)
 
@@ -122,8 +133,7 @@ typedef FT_Int TA_Angle;
             TA_Angle _delta = (angle2) - (angle1); \
 \
 \
-            _delta %= TA_ANGLE_2PI; \
-            if (_delta < 0) \
+            while (_delta <= -TA_ANGLE_PI) \
               _delta += TA_ANGLE_2PI; \
 \
             if (_delta > TA_ANGLE_PI) \
@@ -140,9 +150,11 @@ typedef struct TA_GlyphHintsRec_* TA_GlyphHints;
 
 /* a scaler models the target pixel device that will receive */
 /* the auto-hinted glyph image */
-#define TA_SCALER_FLAG_NO_HORIZONTAL 0x01 /* disable horizontal hinting */
-#define TA_SCALER_FLAG_NO_VERTICAL 0x02 /* disable vertical hinting */
-#define TA_SCALER_FLAG_NO_ADVANCE 0x04 /* disable advance hinting */
+#define TA_SCALER_FLAG_NO_HORIZONTAL 0x01U /* disable horizontal hinting */
+#define TA_SCALER_FLAG_NO_VERTICAL 0x02U /* disable vertical hinting */
+#define TA_SCALER_FLAG_NO_ADVANCE 0x04U /* disable advance hinting */
+#define TA_SCALER_FLAG_NO_WARPER 0x08U /* disable warper */
+
 
 typedef struct TA_ScalerRec_
 {
@@ -178,7 +190,8 @@ typedef FT_Error
 (*TA_WritingSystem_InitHintsFunc)(TA_GlyphHints hints,
                                   TA_StyleMetrics metrics);
 typedef void
-(*TA_WritingSystem_ApplyHintsFunc)(TA_GlyphHints hints,
+(*TA_WritingSystem_ApplyHintsFunc)(FT_UInt glyph_index,
+                                   TA_GlyphHints hints,
                                    FT_Outline* outline,
                                    TA_StyleMetrics metrics);
 
@@ -235,8 +248,9 @@ typedef const TA_WritingSystemClassRec* TA_WritingSystemClass;
 
 
 /*
- * Each script is associated with a set of Unicode ranges that gets used
- * to test whether the font face supports the script.
+ * Each script is associated with two sets of Unicode ranges to test
+ * whether the font face supports the script, and which base characters
+ * the script contains.
  *
  * We use four-letter script tags from the OpenType specification,
  * extended by `NONE', which indicates `no script'.
@@ -270,7 +284,9 @@ typedef struct TA_ScriptClassRec_
 {
   TA_Script script;
 
-  TA_Script_UniRange script_uni_ranges; /* last must be { 0, 0 } */
+  /* last element in the ranges must be { 0, 0 } */
+  TA_Script_UniRange script_uni_ranges;
+  TA_Script_UniRange script_uni_nonbase_ranges;
 
   FT_UInt32 standard_char1; /* for default width and height */
   FT_UInt32 standard_char2; /* ditto */
@@ -317,12 +333,12 @@ typedef const TA_ScriptClassRec* TA_ScriptClass;
 
 #undef COVERAGE
 #define COVERAGE(name, NAME, description, \
-                 tag1, tag2, tag3, tag4) \
+                 tag, tag1, tag2, tag3, tag4) \
           TA_COVERAGE_ ## NAME,
 
 typedef enum TA_Coverage_
 {
-#include "tacover.h"
+#include "ttfautohint-coverages.h"
 
   TA_COVERAGE_DEFAULT
 } TA_Coverage;
